from datetime import datetime
import sys


VERSION = (2, 0, 'b7')

default_app_config = 'dbmail.apps.DBMailConfig'


def get_version():
    return '.'.join(map(str, VERSION))


def app_installed(app):
    from django.conf import settings

    return app in settings.INSTALLED_APPS


def celery_supported():
    try:
        import tasks

        if not app_installed('djcelery'):
            raise ImportError
        return True
    except ImportError:
        return False


def send_db_mail(slug, recipient, *args, **kwargs):
    from dbmail.defaults import CELERY_QUEUE, SEND_MAX_TIME, ENABLE_CELERY
    from dbmail.models import MailTemplate
    from dbmail.send_mail import SendMail

    args = (slug, recipient) + args
    send_after = kwargs.pop('send_after', None)
    send_at_date = kwargs.pop('send_at_date', None)
    use_celery = kwargs.pop('use_celery', ENABLE_CELERY)

    if celery_supported() and use_celery is True:
        import tasks

        template = MailTemplate.get_template(slug=slug)
        max_retries = kwargs.get('max_retries', None)
        if max_retries is None and template.num_of_retries:
            kwargs['max_retries'] = template.num_of_retries

        options = {
            'args': args, 'kwargs': kwargs,
            'queue': kwargs.pop('queue', CELERY_QUEUE),
            'time_limit': kwargs.get('time_limit', SEND_MAX_TIME),
            'priority': template.priority,
        }

        if send_at_date is not None and isinstance(send_at_date, datetime):
            options.update({'eta': send_at_date})
        if send_after is not None:
            options.update({'countdown': send_after})
        if template.is_active:
            return tasks.send_db_mail.apply_async(**options)
    else:
        return SendMail(*args, **kwargs).send(is_celery=False)


def initial_signals():
    from django.db.utils import DatabaseError, IntegrityError

    for cmd in ['schemamigration', 'migrate', 'syncdb',
                'test', 'createsuperuser', 'makemigrations']:
        if cmd in sys.argv:
            break
    else:
        try:
            from dbmail.signals import initial_signals

            initial_signals()
        except (ImportError, DatabaseError, IntegrityError):
            pass



# -*- encoding: utf-8 -*-

import datetime
import uuid
import os

from django.utils.translation import ugettext_lazy as _
from django.utils.html import strip_tags
from django.core.cache import cache
from django.conf import settings
from django.db import models
from django import VERSION

from dbmail.defaults import (
    PRIORITY_STEPS, UPLOAD_TO, DEFAULT_CATEGORY,
    DEFAULT_FROM_EMAIL, DEFAULT_PRIORITY)

from dbmail.utils import premailer_transform
from dbmail.utils import clean_cache_key
from dbmail.fields import HTMLField
from dbmail import initial_signals


def _upload_mail_file(instance, filename):
    ext = filename.split('.')[-1]
    filename = "%s.%s" % (str(uuid.uuid4()), ext)
    return os.path.join(UPLOAD_TO, filename)


class MailCategory(models.Model):
    name = models.CharField(_('Category'), max_length=25, unique=True)
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    updated = models.DateTimeField(_('Updated'), auto_now=True)

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _('Mail category')
        verbose_name_plural = _('Mail categories')


class MailFromEmailCredential(models.Model):
    host = models.CharField(_('Host'), max_length=50)
    port = models.PositiveIntegerField(_('Port'))
    username = models.CharField(
        _('Username'), max_length=50, null=True, blank=True)
    password = models.CharField(
        _('Password'), max_length=50, null=True, blank=True)
    use_tls = models.BooleanField(_('Use TLS'), default=False)
    fail_silently = models.BooleanField(_('Fail silently'), default=False)
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    updated = models.DateTimeField(_('Updated'), auto_now=True)

    def __unicode__(self):
        return '%s/%s' % (self.username, self.host)

    class Meta:
        verbose_name = _('Mail auth settings')
        verbose_name_plural = _('Mail auth settings')


class MailFromEmail(models.Model):
    name = models.CharField(_('Name'), max_length=100)
    email = models.EmailField(_('Email'), unique=True)
    credential = models.ForeignKey(
        MailFromEmailCredential, verbose_name=_('Auth credentials'),
        blank=True, null=True, default=None)
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    updated = models.DateTimeField(_('Updated'), auto_now=True)

    def __unicode__(self):
        return self.name

    @property
    def get_mail_from(self):
        return u'%s <%s>' % (self.name, self.email)

    class Meta:
        verbose_name = _('Mail from')
        verbose_name_plural = _('Mail from')

    def _clean_template_cache(self):
        MailTemplate.clean_cache(from_email__email=self.email)

    def _update_credential_cache(self):
        if not self.credential:
            cache.delete(clean_cache_key(self.get_mail_from), version=1)
        else:
            cache.set(clean_cache_key(self.get_mail_from), dict(
                host=self.credential.host,
                port=self.credential.port,
                username=self.credential.username,
                password=self.credential.password,
                use_tls=self.credential.use_tls,
                fail_silently=self.credential.fail_silently
            ), timeout=None, version=1)

    def save(self, *args, **kwargs):
        self._clean_template_cache()
        self._update_credential_cache()
        return super(MailFromEmail, self).save(*args, **kwargs)


class MailBcc(models.Model):
    email = models.EmailField(_('Email'), unique=True)
    is_active = models.BooleanField(_('Is active'), default=True)
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    updated = models.DateTimeField(_('Updated'), auto_now=True)

    def __unicode__(self):
        return self.email

    class Meta:
        verbose_name = _('Mail Bcc')
        verbose_name_plural = _('Mail Bcc')

    def __clean_cache(self):
        MailTemplate.clean_cache(bcc_email=self)

    def save(self, *args, **kwargs):
        self.__clean_cache()
        return super(MailBcc, self).save(*args, **kwargs)

    def delete(self, using=None):
        self.__clean_cache()
        super(MailBcc, self).delete(using)


class MailTemplate(models.Model):
    name = models.CharField(_('Template name'), max_length=100, db_index=True)
    subject = models.CharField(_('Subject'), max_length=100)
    from_email = models.ForeignKey(
        MailFromEmail, null=True, blank=True,
        verbose_name=_('From email'), default=DEFAULT_FROM_EMAIL,
        help_text=_('If not specified, then used default.'))
    bcc_email = models.ManyToManyField(
        MailBcc, verbose_name=_('Bcc'), blank=True, null=True,
        help_text='Blind carbon copy')
    message = HTMLField(_('Body'))
    slug = models.SlugField(
        _('Slug'), unique=True,
        help_text=_('Unique slug to use in code.'))
    num_of_retries = models.PositiveIntegerField(
        _('Number of retries'), default=1)
    priority = models.SmallIntegerField(
        _('Priority'), default=DEFAULT_PRIORITY, choices=PRIORITY_STEPS)
    is_html = models.BooleanField(_('Is html'), default=True)
    is_admin = models.BooleanField(_('For admin'), default=False)
    is_active = models.BooleanField(_('Is active'), default=True)
    enable_log = models.BooleanField(_('Logging enabled'), default=True)
    category = models.ForeignKey(
        MailCategory, null=True, blank=True,
        verbose_name=_('Category'), default=DEFAULT_CATEGORY)
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    updated = models.DateTimeField(_('Updated'), auto_now=True)
    context_note = models.TextField(
        _('Context note'), null=True, blank=True,
        help_text=_(
            'This is simple note field for context variables with description'
        )
    )

    def _clean_cache(self):
        cache.delete(self.slug, version=1)
        cache.delete(self.slug, version=2)
        cache.delete(self.slug, version=3)

    def _clean_non_html(self):
        if not self.is_html:
            self.message = strip_tags(self.message)
            if hasattr(settings, 'MODELTRANSLATION_LANGUAGES'):
                for lang in settings.MODELTRANSLATION_LANGUAGES:
                    message = strip_tags(getattr(self, 'message_%s' % lang))
                    if message:
                        setattr(self, 'message_%s' % lang, message)

    def _premailer_transform(self):
        if self.is_html:
            self.message = premailer_transform(self.message)

    @classmethod
    def clean_cache(cls, **kwargs):
        for template in cls.objects.filter(**kwargs):
            template._clean_cache()

    def save(self, *args, **kwargs):
        self._clean_cache()
        self._clean_non_html()
        self._premailer_transform()
        return super(MailTemplate, self).save(*args, **kwargs)

    def delete(self, using=None):
        self._clean_cache()
        super(MailTemplate, self).delete(using)

    @property
    def files_list(self):
        files = cache.get(self.slug, version=3)
        if files is None:
            files = list(self.files.all())
            cache.set(self.slug, files, timeout=None, version=3)
        return files

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _('Mail template')
        verbose_name_plural = _('Mail templates')

    @classmethod
    def get_template(cls, slug):
        obj = cache.get(slug, version=1)
        if obj is not None:
            return obj
        else:
            obj = cls.objects.select_related('from_email').get(slug=slug)
            bcc = [o.email for o in obj.bcc_email.filter(is_active=1)]
            cache.set(slug, obj, timeout=None, version=1)
            cache.set(slug, bcc, timeout=None, version=2)
            if obj.from_email:
                obj.from_email._update_credential_cache()
            return obj


class MailFile(models.Model):
    template = models.ForeignKey(
        MailTemplate, verbose_name=_('Template'), related_name='files')
    name = models.CharField(_('Name'), max_length=100)
    filename = models.FileField(_('File'), upload_to=_upload_mail_file)

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _('Mail file')
        verbose_name_plural = _('Mail files')


class MailLogException(models.Model):
    name = models.CharField(_('Exception'), max_length=150, unique=True)

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _('Mail Exception')
        verbose_name_plural = _('Mail Exception')


class MailLog(models.Model):
    is_sent = models.BooleanField(_('Is sent'), default=True, db_index=True)
    template = models.ForeignKey(MailTemplate, verbose_name=_('Template'))
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, verbose_name=_('User'),
        null=True, blank=True)
    error_message = models.TextField(_('Error message'), null=True, blank=True)
    error_exception = models.ForeignKey(
        MailLogException, null=True, blank=True, verbose_name=_('Exception'))
    num_of_retries = models.PositiveIntegerField(
        _('Number of retries'), default=1)

    def __unicode__(self):
        return self.template.name

    class Meta:
        verbose_name = _('Mail log')
        verbose_name_plural = _('Mail logs')

    @staticmethod
    def store_email_log(log, email_list, mail_type):
        if log and email_list:
            for email in email_list:
                MailLogEmail.objects.create(
                    log=log, email=email, mail_type=mail_type
                )

    @classmethod
    def store(cls, to, cc, bcc, is_sent, template, user, num, msg='', ex=None):
        if ex is not None:
            ex = MailLogException.objects.get_or_create(name=ex)[0]

        log = cls.objects.create(
            template=template, is_sent=is_sent, user=user,
            num_of_retries=num, error_message=msg, error_exception=ex
        )
        cls.store_email_log(log, to, 'to')
        cls.store_email_log(log, cc, 'cc')
        cls.store_email_log(log, bcc, 'bcc')

    @classmethod
    def cleanup(cls, days=7):
        date = datetime.datetime.now() - datetime.timedelta(days=days)
        cls.objects.filter(created__lte=date).delete()


class MailLogEmail(models.Model):
    log = models.ForeignKey(MailLog)
    email = models.EmailField()
    mail_type = models.CharField(_('Mail type'), choices=(
        ('cc', 'CC'),
        ('bcc', 'BCC'),
        ('to', 'TO'),
    ), max_length=3)

    def __unicode__(self):
        return self.email

    class Meta:
        verbose_name = _('Mail log email')
        verbose_name_plural = _('Mail log emails')


class MailGroup(models.Model):
    name = models.CharField(_('Group name'), max_length=100)
    slug = models.SlugField(_('Slug'), unique=True)
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    updated = models.DateTimeField(_('Updated'), auto_now=True)

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _('Mail group')
        verbose_name_plural = _('Mail groups')

    def save(self, *args, **kwargs):
        cache.delete(self.slug, version=2)
        return super(MailGroup, self).save(*args, **kwargs)

    def delete(self, using=None):
        cache.delete(self.slug, version=2)
        super(MailGroup, self).delete(using)

    @classmethod
    def get_emails(cls, slug):
        obj = cache.get(slug, version=2)
        if obj is not None:
            return obj

        obj = MailGroupEmail.objects.filter(group__slug=slug)
        cache.set(slug, obj, timeout=None, version=2)
        return obj


class MailGroupEmail(models.Model):
    name = models.CharField(_('Username'), max_length=100)
    email = models.EmailField(_('Email'))
    group = models.ForeignKey(
        MailGroup, verbose_name=_('Group'), related_name='emails')

    def __unicode__(self):
        return self.email

    class Meta:
        verbose_name = _('Mail group email')
        verbose_name_plural = _('Mail group emails')

        unique_together = (('email', 'group',),)

    def save(self, *args, **kwargs):
        cache.delete(self.group.slug, version=2)
        return super(MailGroupEmail, self).save(*args, **kwargs)

    def delete(self, using=None):
        cache.delete(self.group.slug, version=2)
        super(MailGroupEmail, self).delete(using)


class Signal(models.Model):
    SIGNALS = (
        'pre_save',
        'post_save',
        'pre_delete',
        'post_delete',
        'm2m_changed',
    )
    name = models.CharField(_('Name'), max_length=100)
    model = models.ForeignKey(
        'contenttypes.ContentType', verbose_name=_('Model'))
    signal = models.CharField(
        _('Signal'), choices=zip(SIGNALS, SIGNALS),
        max_length=15, default='post_save')
    template = models.ForeignKey(MailTemplate, verbose_name=_('Template'))
    group = models.ForeignKey(
        MailGroup, verbose_name=_('Email group'), blank=True, null=True,
        help_text=_('You can use group email or rules for recipients.'))
    rules = models.TextField(
        help_text=_(
            'Template should return email to send message. Example:'
            '{% if instance.is_active %}{{ instance.email }}{% endif %}.'
            'You can return a multiple emails separated by commas.'
        ), default='{{ instance.email }}', verbose_name=_('Rules'),
        null=True, blank=True
    )
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    updated = models.DateTimeField(_('Updated'), auto_now=True)
    is_active = models.BooleanField(_('Is active'), default=True)
    receive_once = models.BooleanField(
        _('Receive once'), default=True,
        help_text=_('Signal will be receive and send once for new db row.'))
    interval = models.PositiveIntegerField(
        _('Send interval'), default=0,
        help_text=_(
            'Specify interval to send messages after sometime. '
            'That very helpful for mailing on enterprise products.'
            'Interval must be set in the seconds.'
        ))

    def is_sent(self, pk):
        if pk is not None:
            if self.receive_once is True:
                return SignalLog.objects.filter(
                    model=self.model, model_pk=pk, signal=self).exists()
            return False

    def mark_as_sent(self, pk):
        if pk is not None:
            SignalLog.objects.create(
                model=self.model, model_pk=pk, signal=self
            )

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _('Mail signal')
        verbose_name_plural = _('Mail signals')


class SignalLog(models.Model):
    model = models.ForeignKey('contenttypes.ContentType')
    model_pk = models.BigIntegerField()
    signal = models.ForeignKey(Signal)
    created = models.DateTimeField(_('Created'), auto_now_add=True)


class ApiKey(models.Model):
    name = models.CharField(_('Name'), max_length=25)
    api_key = models.CharField(_('Api key'), max_length=32, unique=True)
    is_active = models.BooleanField(_('Is active'), default=True)
    created = models.DateTimeField(_('Created'), auto_now_add=True)
    updated = models.DateTimeField(_('Updated'), auto_now=True)

    def _clean_cache(self):
        cache.delete(self.api_key)

    @classmethod
    def clean_cache(cls):
        for api in cls.objects.all():
            api._clean_cache()

    def save(self, *args, **kwargs):
        self._clean_cache()
        return super(ApiKey, self).save(*args, **kwargs)

    def delete(self, using=None):
        self._clean_cache()
        super(ApiKey, self).delete(using)

    def __unicode__(self):
        return self.name

    class Meta:
        verbose_name = _('Mail API')
        verbose_name_plural = _('Mail API')


if VERSION < (1, 7):
    initial_signals()



# -*- coding: utf-8 -*-

from celery import task
from dbmail.defaults import SEND_RETRY_DELAY, SEND_RETRY, SEND_MAX_TIME


@task(name='dbmail.send_db_mail', default_retry_delay=SEND_RETRY_DELAY)
def send_db_mail(*args, **kwargs):
    from dbmail.send_mail import SendMail

    retry_delay = kwargs.pop('retry_delay', SEND_RETRY_DELAY)
    time_limit = kwargs.pop('time_limit', SEND_MAX_TIME)
    max_retries = kwargs.pop('max_retries', SEND_RETRY)
    retry = kwargs.pop('retry', True)

    try:
        return SendMail(*args, **kwargs).send(is_celery=True)
    except Exception, exc:
        if retry is True and max_retries:
            raise send_db_mail.retry(
                retry=retry, max_retries=max_retries,
                countdown=retry_delay, exc=exc,
                time_limit=time_limit,
            )
        raise


@task(name='dbmail.signal_receiver')
def signal_receiver(*args, **kwargs):
    from dbmail.signals import SignalReceiver

    return SignalReceiver(*args, **kwargs)



# -*- encoding: utf-8 -*-

from django.db import models

from dbmail import app_installed

HTMLField = models.TextField

if app_installed('tinymce'):
    try:
        from tinymce.models import HTMLField

    except ImportError:
        pass



# -*- coding: utf-8 -*-

import os

from django.contrib.staticfiles.templatetags.staticfiles import static
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ImproperlyConfigured
from django.shortcuts import redirect, render
from django.core.urlresolvers import reverse
from django.conf.urls import patterns, url
from django.db.models import get_model
from django.contrib import messages
from django.contrib import admin

from dbmail.models import (
    MailCategory, MailTemplate, MailLog, MailLogEmail, Signal, ApiKey, MailBcc,
    MailGroup, MailGroupEmail, MailFile, MailFromEmail, MailFromEmailCredential
)
from dbmail import app_installed
from dbmail import defaults

ModelAdmin = admin.ModelAdmin

if app_installed('reversion'):
    try:
        from reversion import VersionAdmin

        ModelAdmin = VersionAdmin
    except ImportError:
        pass

if app_installed('reversion_compare'):
    try:
        from reversion_compare.admin import CompareVersionAdmin

        ModelAdmin = CompareVersionAdmin
    except ImportError:
        pass


class MailCategoryAdmin(admin.ModelAdmin):
    list_display = ('name', 'created', 'updated', 'id',)
    list_filter = ('created', 'updated',)
    search_fields = ('name',)


class MailTemplateFileAdmin(admin.TabularInline):
    model = MailFile
    extra = 1


class MailTemplateAdmin(ModelAdmin):
    list_display = (
        'name', 'category', 'from_email', 'slug', 'is_admin', 'is_html',
        'enable_log', 'is_active', 'num_of_retries', 'priority',
        'created', 'updated', 'id',
    )
    list_filter = (
        'category', 'is_active', 'is_admin', 'is_html', 'priority',
        'from_email', 'created', 'updated',)
    search_fields = (
        'name', 'subject', 'slug', 'message',)
    ordering = ('-id',)
    list_editable = ('category', 'priority', 'is_active',)
    list_display_links = ('name',)
    date_hierarchy = 'created'
    list_per_page = defaults.TEMPLATES_PER_PAGE
    inlines = [MailTemplateFileAdmin]

    class Media:
        try:
            js = (
                static('dbmail/admin/js/dbmail.js'),
            )
        except ImproperlyConfigured:
            js = (
                '/media/dbmail/admin/js/dbmail.js',
                '/static/dbmail/admin/js/dbmail.js',
            )

    def send_mail_view(self, request, pk):
        from dbmail.management.commands.dbmail_test_send import send_test_msg

        if request.user.email:
            send_test_msg(pk, request.user.email, request.user)
        else:
            messages.error(
                request, 'Set your email address in user settings.'
            )

        return redirect(
            reverse(
                'admin:dbmail_mailtemplate_change', args=(pk,),
                current_app=self.admin_site.name
            )
        )

    def get_apps_view(self, request, pk):
        apps_list = {}
        for ct in ContentType.objects.all():
            if ct.app_label not in apps_list:
                apps_list[ct.app_label] = [ct]
            elif ct.model_class():
                apps_list[ct.app_label].append(ct)
        return render(request, 'dbmail/apps.html', {'apps_list': apps_list})

    def browse_model_fields_view(self, request, pk, app, model):
        fields = dict()
        if pk and get_model(app, model):
            for f in get_model(app, model)._meta.fields:
                fields[f.name] = unicode(f.verbose_name)
        return render(request, 'dbmail/browse.html', {'fields_list': fields})

    def get_urls(self):
        urls = super(MailTemplateAdmin, self).get_urls()
        admin_urls = patterns(
            '',
            url(
                r'^(\d+)/sendmail/$',
                self.admin_site.admin_view(self.send_mail_view),
                name='send_mail_view'
            ),
            url(
                r'^(\d+)/sendmail/apps/(.*?)/(.*?)/',
                self.admin_site.admin_view(self.browse_model_fields_view),
                name='browse_model_fields_view'),
            url(
                r'^(\d+)/sendmail/apps/',
                self.admin_site.admin_view(self.get_apps_view),
                name='send_mail_apps_view'
            ),
        )
        return admin_urls + urls

    def get_form(self, request, obj=None, **kwargs):
        if obj is not None:
            self.prepopulated_fields = {}
            if defaults.READ_ONLY_ENABLED:
                self.readonly_fields = ['slug', 'context_note']
        else:
            self.prepopulated_fields = {'slug': ('name',)}

        return super(MailTemplateAdmin, self).get_form(
            request, obj=None, **kwargs)


class MailLogEmailInline(admin.TabularInline):
    readonly_fields = [field.name for field in MailLogEmail._meta.fields]
    model = MailLogEmail
    extra = 0

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        if not request.user.is_superuser:
            return False
        return True

    def has_change_permission(self, request, obj=None):
        return request.method != 'POST'


class MailLogAdmin(admin.ModelAdmin):
    list_display = (
        'template', 'created', 'is_sent', 'num_of_retries', 'user', 'id',)
    list_filter = ('is_sent', 'created', 'error_exception', 'template',)
    date_hierarchy = 'created'
    inlines = [MailLogEmailInline]
    search_fields = ('maillogemail__email', 'user__username', 'user__email',)

    def __init__(self, model, admin_site):
        super(MailLogAdmin, self).__init__(model, admin_site)

        self.readonly_fields = [field.name for field in model._meta.fields]
        self.readonly_model = model

    def has_add_permission(self, request):
        return False

    def has_delete_permission(self, request, obj=None):
        if not request.user.is_superuser:
            return False
        return True

    def has_change_permission(self, request, obj=None):
        return request.method != 'POST'


class MailGroupEmailInline(admin.TabularInline):
    model = MailGroupEmail
    extra = 1


class MailGroupAdmin(admin.ModelAdmin):
    list_display = ('name', 'slug', 'created', 'updated', 'id',)
    list_filter = ('updated', 'created',)
    prepopulated_fields = {'slug': ('name',)}
    inlines = [MailGroupEmailInline]

    def get_readonly_fields(self, request, obj=None):
        if obj is not None and defaults.READ_ONLY_ENABLED:
            return ['slug']
        return self.readonly_fields

    def get_prepopulated_fields(self, request, obj=None):
        if obj is not None:
            return {}
        return self.prepopulated_fields


class MailFromEmailAdmin(admin.ModelAdmin):
    list_display = ('name', 'email', 'credential', 'created', 'updated', 'id',)
    list_filter = ('updated', 'created',)


class SignalAdmin(admin.ModelAdmin):
    list_display = (
        'name', 'model', 'signal', 'template', 'interval', 'receive_once',
        'updated', 'created', 'id',)
    list_filter = ('signal', 'receive_once', 'updated', 'created',)

    @staticmethod
    def auto_reload(request):
        if defaults.WSGI_AUTO_RELOAD is True:
            env = request.environ.get
            if env('mod_wsgi.process_group') and env('SCRIPT_FILENAME'):
                if int(env('mod_wsgi.script_reloading', 0)):
                    try:
                        if os.path.exists(env('SCRIPT_FILENAME')):
                            os.utime(env('SCRIPT_FILENAME'), None)
                    except OSError:
                        pass

        if defaults.UWSGI_AUTO_RELOAD is True:
            try:
                import uwsgi

                uwsgi.reload()
            except ImportError:
                pass

    def save_model(self, request, *args, **kwargs):
        super(SignalAdmin, self).save_model(request, *args, **kwargs)
        self.auto_reload(request)


class MailFromEmailCredentialAdmin(admin.ModelAdmin):
    list_display = (
        'host', 'port', 'username', 'use_tls',
        'fail_silently', 'updated', 'created', 'id',)
    list_filter = ('use_tls', 'fail_silently', 'updated', 'created',)


class ApiKeyAdmin(admin.ModelAdmin):
    list_display = (
        'name', 'api_key', 'is_active', 'updated', 'created', 'id',)
    list_filter = ('is_active', 'updated', 'created',)


class MailBccAdmin(admin.ModelAdmin):
    list_display = (
        'email', 'is_active', 'updated', 'created', 'id',)
    list_filter = ('is_active', 'updated', 'created',)


def admin_register(model):
    model_name = model.__name__
    if model_name in defaults.ALLOWED_MODELS_ON_ADMIN:
        admin_cls = globals().get('%sAdmin' % model_name)
        if admin_cls:
            admin.site.register(model, admin_cls)
        else:
            admin.site.register(model)


admin_register(MailFromEmailCredential)
admin_register(MailFromEmail)
admin_register(MailCategory)
admin_register(MailTemplate)
admin_register(MailLog)
admin_register(MailGroup)
admin_register(Signal)
admin_register(ApiKey)
admin_register(MailBcc)



from django.core.management.base import BaseCommand
from dbmail.models import MailTemplate, ApiKey


class Command(BaseCommand):
    def handle(self, *args, **options):
        MailTemplate.clean_cache()
        ApiKey.clean_cache()



# -*- encoding: utf-8 -*-

from django.utils.translation import ugettext_lazy as _
from django.conf import settings


def get_settings(key, default):
    return getattr(settings, key, default)


PRIORITY_STEPS = get_settings('DB_MAILER_PRIORITY_STEPS', (
    (0, _("High")),
    (3, _("Medium")),
    (6, _("Low")),
    (9, _("Deferred")),
))
CELERY_QUEUE = get_settings('DB_MAILER_CELERY_QUEUE', 'default')
ENABLE_CELERY = get_settings('DB_MAILER_ENABLE_CELERY', True)
SHOW_CONTEXT = get_settings('DB_MAILER_SHOW_CONTEXT', False)
READ_ONLY_ENABLED = get_settings('DB_MAILER_READ_ONLY_ENABLED', True)
UPLOAD_TO = get_settings('DB_MAILER_UPLOAD_TO', 'mail_files')
DEFAULT_CATEGORY = get_settings('DB_MAILER_DEFAULT_CATEGORY', None)
DEFAULT_FROM_EMAIL = get_settings('DB_MAILER_DEFAULT_FROM_EMAIL', None)
DEFAULT_PRIORITY = get_settings('DB_MAILER_DEFAULT_PRIORITY', 6)
TEMPLATES_PER_PAGE = get_settings('DB_MAILER_TEMPLATES_PER_PAGE', 20)
SEND_RETRY = get_settings('DB_MAILER_SEND_RETRY', 3)
SEND_RETRY_DELAY = get_settings('DB_MAILER_SEND_RETRY_DELAY', 300)
SEND_RETRY_DELAY_DIRECT = get_settings('DB_MAILER_SEND_RETRY_DELAY_DIRECT', 6)
SEND_MAX_TIME = get_settings('DB_MAILER_SEND_MAX_TIME', 30)
WSGI_AUTO_RELOAD = get_settings('DB_MAILER_WSGI_AUTO_RELOAD', False)
UWSGI_AUTO_RELOAD = get_settings('DB_MAILER_UWSGI_AUTO_RELOAD', False)
ENABLE_LOGGING = get_settings('DB_MAILER_ENABLE_LOGGING', True)
ADD_HEADER = get_settings('DB_MAILER_ADD_HEADER', False)
LOGS_EXPIRE_DAYS = get_settings('DB_MAILER_LOGS_EXPIRE_DAYS', 7)
ALLOWED_MODELS_ON_ADMIN = get_settings('DB_MAILER_ALLOWED_MODELS_ON_ADMIN', [
    'MailFromEmailCredential',
    'MailFromEmail',
    'MailCategory',
    'MailTemplate',
    'MailLog',
    'MailGroup',
    'Signal',
    'ApiKey',
    'MailBcc',
])



# -*- encoding: utf-8 -*-

import json

from django.views.decorators.csrf import csrf_exempt
from django.shortcuts import get_object_or_404
from django.http import HttpResponse, Http404
from django.core.cache import cache

from dbmail.models import ApiKey
from dbmail import send_db_mail

allowed_fields = [
    'api_key', 'slug', 'recipient', 'from_email', 'cc', 'bcc', 'queue',
    'retry_delay', 'max_retries', 'retry', 'time_limit', 'send_after',
]


@csrf_exempt
def send_by_dbmail(request):
    if request.method == 'POST':
        kwargs = dict()
        for f in allowed_fields:
            if request.POST.get(f):
                kwargs[f] = request.POST.get(f)

        api_key = kwargs.get('api_key')
        if api_key:
            del kwargs['api_key']
            if not cache.get(api_key):
                get_object_or_404(
                    ApiKey, api_key=api_key, is_active=True)
                cache.set(api_key, 1, timeout=None)

            args = []
            if request.POST.get('data'):
                args = [json.loads(request.POST['data'])]

            if kwargs.get('slug') and kwargs.get('recipient'):
                send_db_mail(
                    kwargs.pop('slug'), kwargs.pop('recipient'),
                    *args, **kwargs)
                return HttpResponse('OK')
    raise Http404



# -*- encoding: utf-8 -*-

from modeltranslation.translator import translator, TranslationOptions
from dbmail.models import MailTemplate


class MailTemplateTranslationOptions(TranslationOptions):
    fields = ('subject', 'message',)

translator.register(MailTemplate, MailTemplateTranslationOptions)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings
import dbmail.models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ApiKey',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=25, verbose_name='Name')),
                ('api_key', models.CharField(unique=True, max_length=32, verbose_name='Api key')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is active')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
            ],
            options={
                'verbose_name': 'Mail API',
                'verbose_name_plural': 'Mail API',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MailBcc',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('email', models.EmailField(unique=True, max_length=75, verbose_name='Email')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is active')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
            ],
            options={
                'verbose_name': 'Mail Bcc',
                'verbose_name_plural': 'Mail Bcc',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MailCategory',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(unique=True, max_length=25, verbose_name='Category')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
            ],
            options={
                'verbose_name': 'Mail category',
                'verbose_name_plural': 'Mail categories',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MailFile',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('filename', models.FileField(upload_to=dbmail.models._upload_mail_file, verbose_name='File')),
            ],
            options={
                'verbose_name': 'Mail file',
                'verbose_name_plural': 'Mail files',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MailFromEmail',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('email', models.EmailField(unique=True, max_length=75, verbose_name='Email')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
            ],
            options={
                'verbose_name': 'Mail from',
                'verbose_name_plural': 'Mail from',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MailFromEmailCredential',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('host', models.CharField(max_length=50, verbose_name='Host')),
                ('port', models.PositiveIntegerField(verbose_name='Port')),
                ('username', models.CharField(max_length=50, null=True, verbose_name='Username', blank=True)),
                ('password', models.CharField(max_length=50, null=True, verbose_name='Password', blank=True)),
                ('use_tls', models.BooleanField(default=False, verbose_name='Use TLS')),
                ('fail_silently', models.BooleanField(default=False, verbose_name='Fail silently')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
            ],
            options={
                'verbose_name': 'Mail auth settings',
                'verbose_name_plural': 'Mail auth settings',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MailGroup',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='Group name')),
                ('slug', models.SlugField(unique=True, verbose_name='Slug')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
            ],
            options={
                'verbose_name': 'Mail group',
                'verbose_name_plural': 'Mail groups',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MailGroupEmail',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='Username')),
                ('email', models.EmailField(max_length=75, verbose_name='Email')),
                ('group', models.ForeignKey(related_name=b'emails', verbose_name='Group', to='dbmail.MailGroup')),
            ],
            options={
                'verbose_name': 'Mail group email',
                'verbose_name_plural': 'Mail group emails',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MailLog',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('is_sent', models.BooleanField(default=True, db_index=True, verbose_name='Is sent')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('error_message', models.TextField(null=True, verbose_name='Error message', blank=True)),
                ('num_of_retries', models.PositiveIntegerField(default=1, verbose_name='Number of retries')),
            ],
            options={
                'verbose_name': 'Mail log',
                'verbose_name_plural': 'Mail logs',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MailLogEmail',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('email', models.EmailField(max_length=75)),
                ('mail_type', models.CharField(max_length=3, verbose_name='Mail type', choices=[(b'cc', b'CC'), (b'bcc', b'BCC'), (b'to', b'TO')])),
                ('log', models.ForeignKey(to='dbmail.MailLog')),
            ],
            options={
                'verbose_name': 'Mail log email',
                'verbose_name_plural': 'Mail log emails',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MailLogException',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(unique=True, max_length=150, verbose_name='Exception')),
            ],
            options={
                'verbose_name': 'Mail Exception',
                'verbose_name_plural': 'Mail Exception',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='MailTemplate',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='Template name', db_index=True)),
                ('subject', models.CharField(max_length=100, verbose_name='Subject')),
                ('message', models.TextField(verbose_name='Body')),
                ('slug', models.SlugField(help_text='Unique slug to use in code.', unique=True, verbose_name='Slug')),
                ('num_of_retries', models.PositiveIntegerField(default=1, verbose_name='Number of retries')),
                ('priority', models.SmallIntegerField(default=6, verbose_name='Priority', choices=[(0, 'High'), (3, 'Medium'), (6, 'Low'), (9, 'Deferred')])),
                ('is_html', models.BooleanField(default=True, verbose_name='Is html')),
                ('is_admin', models.BooleanField(default=False, verbose_name='For admin')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is active')),
                ('enable_log', models.BooleanField(default=True, verbose_name='Logging enabled')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('context_note', models.TextField(help_text='This is simple note field for context variables with description', null=True, verbose_name='Context note', blank=True)),
                ('bcc_email', models.ManyToManyField(help_text=b'Blind carbon copy', to='dbmail.MailBcc', null=True, verbose_name='Bcc', blank=True)),
                ('category', models.ForeignKey(default=None, blank=True, to='dbmail.MailCategory', null=True, verbose_name='Category')),
                ('from_email', models.ForeignKey(default=None, to='dbmail.MailFromEmail', blank=True, help_text='If not specified, then used default.', null=True, verbose_name='From email')),
            ],
            options={
                'verbose_name': 'Mail template',
                'verbose_name_plural': 'Mail templates',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='Signal',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('signal', models.CharField(default=b'post_save', max_length=15, verbose_name='Signal', choices=[(b'pre_save', b'pre_save'), (b'post_save', b'post_save'), (b'pre_delete', b'pre_delete'), (b'post_delete', b'post_delete'), (b'm2m_changed', b'm2m_changed')])),
                ('rules', models.TextField(default=b'{{ instance.email }}', help_text='Template should return email to send message. Example:{% if instance.is_active %}{{ instance.email }}{% endif %}.You can return a multiple emails separated by commas.', null=True, verbose_name='Rules', blank=True)),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is active')),
                ('receive_once', models.BooleanField(default=True, help_text='Signal will be receive and send once for new db row.', verbose_name='Receive once')),
                ('interval', models.PositiveIntegerField(default=0, help_text='Specify interval to send messages after sometime. That very helpful for mailing on enterprise products.Interval must be set in the seconds.', verbose_name='Send interval')),
                ('group', models.ForeignKey(blank=True, to='dbmail.MailGroup', help_text='You can use group email or rules for recipients.', null=True, verbose_name='Email group')),
                ('model', models.ForeignKey(verbose_name='Model', to='contenttypes.ContentType')),
                ('template', models.ForeignKey(verbose_name='Template', to='dbmail.MailTemplate')),
            ],
            options={
                'verbose_name': 'Mail signal',
                'verbose_name_plural': 'Mail signals',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='SignalLog',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('model_pk', models.BigIntegerField()),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('model', models.ForeignKey(to='contenttypes.ContentType')),
                ('signal', models.ForeignKey(to='dbmail.Signal')),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.AddField(
            model_name='maillog',
            name='error_exception',
            field=models.ForeignKey(verbose_name='Exception', blank=True, to='dbmail.MailLogException', null=True),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='maillog',
            name='template',
            field=models.ForeignKey(verbose_name='Template', to='dbmail.MailTemplate'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='maillog',
            name='user',
            field=models.ForeignKey(verbose_name='User', blank=True, to=settings.AUTH_USER_MODEL, null=True),
            preserve_default=True,
        ),
        migrations.AlterUniqueTogether(
            name='mailgroupemail',
            unique_together=set([('email', 'group')]),
        ),
        migrations.AddField(
            model_name='mailfromemail',
            name='credential',
            field=models.ForeignKey(default=None, blank=True, to='dbmail.MailFromEmailCredential', null=True, verbose_name='Auth credentials'),
            preserve_default=True,
        ),
        migrations.AddField(
            model_name='mailfile',
            name='template',
            field=models.ForeignKey(related_name=b'files', verbose_name='Template', to='dbmail.MailTemplate'),
            preserve_default=True,
        ),
    ]



# -*- encoding: utf-8 -*-

import datetime

from django.template import Template, Context
from django.contrib.sites.models import Site
from django.contrib.auth.models import User
from django.db.models import signals

from dbmail.defaults import SEND_RETRY, SEND_RETRY_DELAY
from dbmail.models import Signal, MailGroup
from dbmail.defaults import CELERY_QUEUE


class SignalReceiver(object):
    def __init__(self, sender, **kwargs):
        self.sender = sender
        self.kwargs = kwargs
        self.site = Site.objects.get_current()
        self.instance = kwargs.get('instance')
        self.pk = self.instance and self.instance.pk or None
        self.signal = None

        self.kwargs['old_instance'] = self.get_old_instance()
        self.kwargs['users'] = self.get_users()
        self.kwargs['date'] = datetime.date.today()
        self.kwargs['date_time'] = datetime.datetime.now()

        self.run()

    def get_signal_list(self):
        return Signal.objects.filter(
            model__model=self.sender._meta.module_name,
            is_active=True
        )

    def get_email_list(self):
        if self.signal.group:
            return MailGroup.get_emails(self.signal.group.slug)

        email_list = Template(self.signal.rules).render(Context(self.kwargs))
        return email_list.strip().replace('\r', '').replace('\n', '')

    def get_interval(self):
        options = dict()
        if self.signal.interval >= 0:
            options['send_after'] = self.signal.interval
        return options

    @staticmethod
    def get_users():
        return User.objects.filter(
            is_active=True, is_staff=False, is_superuser=False)

    def get_old_instance(self):
        instance = self.kwargs.get('instance')
        if instance and instance.pk:
            return self.sender.objects.get(
                pk=self.kwargs['instance'].pk)

    def send_mail(self):
        from dbmail import send_db_mail

        email_list = self.get_email_list()
        if email_list and not self.signal.is_sent(self.pk):
            send_db_mail(
                self.signal.template.slug, email_list, self.site,
                self.kwargs, self.instance, **self.get_interval()
            )
            self.signal.mark_as_sent(self.pk)

    def run(self):
        for self.signal in self.get_signal_list():
            self.send_mail()


def signal_receiver(sender, **kwargs):
    from dbmail import celery_supported

    if 'signal' in kwargs:
        kwargs.pop('signal')

    if celery_supported():
        import tasks

        tasks.signal_receiver.apply_async(
            args=[sender], kwargs=kwargs,
            default_retry_delay=SEND_RETRY_DELAY,
            max_retries=SEND_RETRY,
            queue=CELERY_QUEUE,
        )
    else:
        SignalReceiver(sender, **kwargs)


def initial_signals():
    for signal in Signal.objects.filter(is_active=True):
        def_signal = getattr(signals, signal.signal)
        def_signal.connect(
            signal_receiver, sender=signal.model.model_class(),
            dispatch_uid=signal.model.name
        )



from setuptools import setup, find_packages
from dbmail import get_version


setup(
    name='django-db-mailer',
    version=get_version(),
    description='Django module to easily send emails using '
                'django templates stored on database.',
    keywords="django db mail email html text templates mailer",
    long_description=open('README.rst').read(),
    author="GoTLiuM InSPiRiT",
    author_email='gotlium@gmail.com',
    url='http://github.com/LPgenerator/django-db-mailer/',
    packages=find_packages(exclude=['demo']),
    package_data={'dbmail': [
        'locale/*/LC_MESSAGES/django.*',
        'static/dbmail/admin/js/*.js',
    ]},
    include_package_data=True,
    install_requires=[
        'setuptools',
    ],
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
)



# -*- encoding: utf-8 -*-


def clean_cache_key(key):
    return key.replace('>', '').replace('<', '').replace(' ', '')


def premailer_transform(text):
    try:
        from premailer import transform

        return transform(text)
    except Exception, msg:
        print msg.__str__()
        return text



# -*- encoding: utf-8 -*-

import traceback
import pprint
import time

from django.db.models.fields.related import ManyToManyField, ForeignKey
from django.core.mail import EmailMessage, EmailMultiAlternatives
from django.contrib.sites.models import Site
from django.template import Template, Context
from django.core.mail import get_connection
from django.utils.html import strip_tags
from django.utils import translation
from django.core.cache import cache
from django.conf import settings

from dbmail.defaults import SHOW_CONTEXT, ENABLE_LOGGING, ADD_HEADER
from dbmail.models import MailTemplate, MailLog, MailGroup
from dbmail.utils import clean_cache_key
from dbmail import get_version
from dbmail import defaults


class SendMail(object):
    def __init__(self, slug, recipient, *args, **kwargs):
        self._slug = slug

        self._recipient_list = self.__get_recipient_list(recipient)
        self._cc = self.__email_to_list(kwargs.pop('cc', None))
        self._bcc = self.__email_to_list(kwargs.pop('bcc', None))
        self._user = kwargs.pop('user', None)
        self._language = kwargs.pop('language', None)

        self._template = self.__get_template()
        self._context = self.__get_context(args)

        self._subject = self.__get_subject()
        self._message = self.__get_message()
        self._files = kwargs.pop('files', [])
        self._kwargs = kwargs
        self._num = 1
        self._err_msg = None
        self._err_exc = None

        self._kwargs.pop('retry', None)
        self._kwargs.pop('max_retries', None)
        self._kwargs.pop('retry_delay', None)

        self._from_email = self.__get_from_email()
        self.__update_bcc_from_template_settings()
        self.__insert_mailer_identification_head()

    def __insert_mailer_identification_head(self):
        if not ADD_HEADER:
            return
        headers = self._kwargs.pop('headers', {})
        headers.update(
            {'X-Mailer-Wrapper': 'django-db-mailer ver %s' % get_version()})
        self._kwargs['headers'] = headers

    def __get_connection(self):
        auth_credentials = cache.get(
            clean_cache_key(self._from_email), version=1)
        if auth_credentials:
            return self._kwargs.pop('connection', None) or get_connection(
                **auth_credentials)
        return self._kwargs.pop('connection', None)

    def __get_template(self):
        return MailTemplate.get_template(slug=self._slug)

    def __get_context(self, context_list):
        try:
            data = self.__model_to_dict(Site.objects.get_current())
        except Site.DoesNotExist:
            data = {}

        for context in context_list:
            if isinstance(context, dict):
                data.update(context)
            elif hasattr(context, '_meta'):
                data.update(self.__model_to_dict(context))
                data.update({context._meta.module_name: context})

        if settings.DEBUG and SHOW_CONTEXT:
            pprint.pprint(data)
        return data

    def __get_str_by_language(self, field):
        template = getattr(self._template, field)
        if self._language is not None:
            field = '%s_%s' % (field, self._language)
            if hasattr(self._template, field):
                if getattr(self._template, field):
                    template = getattr(self._template, field)
        return template

    def __get_subject(self):
        return self.__render_template(
            self.__get_str_by_language('subject'), self._context)

    def __get_message(self):
        return self.__render_template(
            self.__get_str_by_language('message'), self._context)

    def __attach_files(self, mail):
        for file_object in self._template.files_list:
            mail.attach_file(file_object.filename.path)

        for filename in self._files:
            mail.attach_file(filename)

    def __send_html_message(self):
        msg = EmailMultiAlternatives(
            self._subject, strip_tags(self._message), cc=self._cc,
            from_email=self._from_email, to=self._recipient_list,
            bcc=self._bcc, connection=self.__get_connection(), **self._kwargs
        )
        msg.attach_alternative(self._message, "text/html")
        self.__attach_files(msg)
        msg.send()

    def __send_plain_message(self):
        msg = EmailMessage(
            self._subject, self._message, from_email=self._from_email,
            to=self._recipient_list, cc=self._cc, bcc=self._bcc,
            connection=self.__get_connection(), **self._kwargs
        )
        self.__attach_files(msg)
        msg.send()

    def __get_recipient_list(self, recipient):
        if not isinstance(recipient, list) and '@' not in recipient:
            return self.__group_emails(recipient)
        return self.__email_to_list(recipient)

    def __update_bcc_from_template_settings(self):
        template_bcc = cache.get(self._slug, version=2)
        if template_bcc is not None:
            if self._bcc:
                self._bcc.extend(template_bcc)
            else:
                self._bcc = template_bcc

    def __get_from_email(self):
        if self._kwargs.get('from_email'):
            return self._kwargs.pop('from_email', None)
        elif not self._template.from_email:
            return settings.DEFAULT_FROM_EMAIL
        return self._template.from_email.get_mail_from

    @staticmethod
    def __group_emails(recipient):
        email_list = []
        for slug in recipient.split(','):
            for obj in MailGroup.get_emails(slug):
                email_list.append(obj.email.strip())
        return email_list

    @staticmethod
    def __email_to_list(recipient):
        if recipient is None:
            return None
        elif not isinstance(recipient, list):
            recipient = [d.strip() for d in recipient.split(',') if d.strip()]
        return recipient

    def __render_template(self, template, context):
        translation.activate(self._language or settings.LANGUAGE_CODE)
        return Template(template).render(Context(context))

    @staticmethod
    def __model_to_dict(instance):
        opts, data = getattr(instance, '_meta'), dict()
        for f in opts.fields + opts.many_to_many:
            if isinstance(f, ManyToManyField):
                if instance.pk is None:
                    data[f.name] = []
                else:
                    data[f.name] = list(f.value_from_object(
                        instance).values_list('pk', flat=True))
            elif isinstance(f, ForeignKey):
                if getattr(instance, f.name):
                    data[f.name] = getattr(instance, f.name).__unicode__()
            else:
                data[f.name] = f.value_from_object(instance)
        return data

    def __send(self):
        if self._template.is_html:
            return self.__send_html_message()
        return self.__send_plain_message()

    def __store_log(self, is_sent):
        if ENABLE_LOGGING is True:
            if self._template.enable_log or not is_sent:
                MailLog.store(
                    self._recipient_list, self._cc, self._bcc,
                    is_sent, self._template, self._user,
                    self._num, self._err_msg, self._err_exc
                )

    def __try_to_send(self):
        for self._num in range(1, self._template.num_of_retries + 1):
            try:
                self.__send()
                break
            except Exception, msg:
                print '[dbmail]', msg.__unicode__()
                if self._template.num_of_retries == self._num:
                    raise
                time.sleep(defaults.SEND_RETRY_DELAY_DIRECT)

    def send(self, is_celery=True):
        if self._template.is_active:
            try:
                if is_celery is True:
                    self.__send()
                else:
                    self.__try_to_send()
                self.__store_log(True)
                return 'OK'
            except Exception as exc:
                self._err_msg = traceback.format_exc()
                self._err_exc = exc.__class__.__name__
                self.__store_log(False)
                raise



import re
import optparse

from django.core.management.base import BaseCommand

from dbmail.models import MailTemplate
from dbmail import send_db_mail


def send_test_msg(pk, email, user=None, use_celery=True):
    template = MailTemplate.objects.get(pk=pk)
    slug = template.slug
    var_list = re.findall('\{\{\s?(\w+)\s?\}\}', template.message)
    context = {}
    for var in var_list:
        context[var] = '%s' % var.upper().replace('_', '-')
    return send_db_mail(slug, email, user, context, use_celery=use_celery)


class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        optparse.make_option('--email', dest='email', help='Recipients'),
        optparse.make_option('--pk', dest='pk', help='DBMail template id'),
        optparse.make_option('--without-celery', action='store_true',
                             default=False, dest='celery',
                             help='Send direct message'),
    )

    def handle(self, *args, **options):
        send_test_msg(
            options['pk'], options['email'], use_celery=not options['celery']
        )
        print "Done. Message was sent."
